clear all
close all
clc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set(0,'DefaultFigureWindowStyle','docked') %%%% Figures 'normal' or 'docked'
Lw0 = 0.6; Lw1 = 1.2; Lw2 = 2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plase choose the configuration you want to use
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% E.g. with reference to Table 3 in the paper:
% nc=3; m=4; m_ii=1 --> Vm=[3 2 1]'; 
% nc=3; m=4; m_ii=2 --> Vm=[3 1 1]'; 
% nc=3; m=4; m_ii=3 --> Vm=[3 2 2]'; 
% nc=3; m=5; m_ii=1 --> Vm=[4 3 2]'; 
% etc ........
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nc    = 4;              % Number of capacitors (to be set equal to 3 or 4)
m     = 16;             % Number of output voltage levels
m_ii  = 1;              % Configuration number among those giving m output voltage levels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load(['GA_All_' num2str(nc) '_C_scaled'],'GA');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nr_Lv = [GA.m];         % Number of output voltage levels for the different configurations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ns    = nc;             % Maximum level-to-level distance for the control algorithm "Distance_n"
RV0   = 0.01;           % Basic radius for the control algorithm "Distance_n"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ind_m = find(Nr_Lv==m);
jj    = Ind_m(m_ii);    % Identification of the chosen configuration
GA_jj = GA(jj);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sm    = GA_jj.Sm;                  % Vectors Sm for the given number of capacitors nc
Lv    = GA_jj.Lv;                  % Output voltage levels associated with each Sm
Vm    = GA_jj.Vm;                  % Configuration of vector Vm=[m-1 alpha_1 ... alpha_nc]'
Vin   = 100;                         % DC input voltage
Rin   = 0.0001;                    % Resistance of the input stage
Cn    = 1*10^-4;                         % Capacitance of capacitor Cnc
T_PWM = 5*10^-5;                   % Period of the saw-tooth carrier
Tc    = 5*10^-7;                   % Sampling time when saving variables in the workspace
Vm0   = Vin*Vm/Vm(1);              % Reference capacitors voltages for the chosen configuration
Vc_0  = Vm0;                       % Initial capacitors voltages
Cm    = Cn*Vm(end)./Vm;            % Capacitance values of the capacitors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plase choose an output voltage signal:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vd_Iout = {{{'Sinusoidal','Bias=0.5','A=0.5','W=2*pi/Tend','Ph=0'}},...
%           {{'Sinusoidal','Bias=1','A=0','W=2*pi/Tend','Ph=0'}},...
%           {{'Tend=0.02','Nr_Punti=2000'}}};
% %( 2) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vd_Iout = {{{'Triangular','Bias=0.5','A=0.5','W=2*pi/Tend','Ph=0'}},...
%           {{'Constant'  ,'Bias=1'}},...
%           {{'Tend=0.02','Nr_Punti=2000'}}};
% % %( 3) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vd_Iout = {{{'SawTooth','Bias=0.5','A=0.5','W=2*pi/Tend','Ph=0'}},...
%           {{'Constant'  ,'Bias=1'}},...
%           {{'Tend=0.02','Nr_Punti=2000'}}};
% %( 4) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Vd_Iout = {{{'Sinusoidal','Bias=0.5','A=0.5','W=2*pi/Tend','Ph=0'}},...
    {{'Sinusoidal','Bias=0.5','A=0.5','W=2*pi/Tend','Ph=0'}},...
    {{'Tend=0.02','Nr_Punti=2000'}}};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[Vd_t_in,~]      = GENERA_SEGNALE(Vd_Iout(1),Vd_Iout(3));   % Generation of the normalized desired voltage signal
[Iout_t_in,t_in] = GENERA_SEGNALE(Vd_Iout(2),Vd_Iout(3));   % Generation of the load current signal
Tfin             = t_in(end);                               % Duration of the simulation   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
color='bm'; 
LineSt='-'; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NrFig            = 1;
Fig_Name         = 'V_and_I_Signals';
hh               = figure(NrFig); clf;
set(hh,'NumberTitle','off') 
set(hh,'Name',[num2str(NrFig) '.' Fig_Name])
set(hh,'PaperPositionMode','manual','PaperPosition',[5 0 20 10]);
%%%%%%%%%%%%
subplot(211);
hold on; grid on; zoom on;
plot(t_in,Vd_t_in*Vin,'g','LineWidth',Lw1)                  % Desired output voltage 
ylabel('Desired voltage Vd\_t')
%%%%%%%%%%%%
subplot(212);
hold on; grid on; zoom on;
plot(t_in,Iout_t_in,'b','LineWidth',Lw1)                    % Load current
ylabel('Load current Iout\_t [A]')
xlabel('Time [s]')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NrFig    = 2;
Fig_Name = 'Flying_C_Voltages';
hh       = figure(NrFig); clf;
set(hh,'NumberTitle','off')
set(hh,'Name',[num2str(NrFig) '.' Fig_Name])
set(hh,'PaperPositionMode','manual','PaperPosition',[5 0 20 10]);
%%%%%%%%%%%%
subplot(1,2,1); hold on        
if nc==3
    plot(Vm0(2),Vm0(3),'.g','MarkerSize',25)
elseif nc==4
%     plot3(x0,y0,z0,'or')
    plot3(Vm0(2),Vm0(3),Vm0(4),'.g','MarkerSize',25)
    view(72,30)
    zlabel('V4')
end
xlabel('V2')
ylabel('V3')
grid on
axis equal
title(['Vm = [' num2str(Vm(1:nc)') ']: Traj. Vc Ctrl 0 (' color(1) ') and Ctrl n (' color(2) ')'])
%%%%%%%%%%%%
subplot(1,2,2); hold on         
grid on
ylabel('DV [V]')
xlabel('Time [s]')
title(['Vm = [' num2str(Vm(1:nc)') ']: Norm |DV| Ctrl 0 (' color(1) ') and Ctrl n (' color(2) ')'])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NrFig    = 3;
Fig_Name = 'Output_Voltage_Error';
hh       = figure(NrFig); clf;
set(hh,'NumberTitle','off') % Clear the title
set(hh,'Name',[num2str(NrFig) '.' Fig_Name])
set(hh,'PaperPositionMode','manual','PaperPosition',[5 0 20 10]);
%%%%%%%%%%%%
subplot(2,1,1);	       
plot(t_in,Vd_t_in*Vin,'g','LineWidth',Lw1);
title(['Vm = [' num2str(Vm(1:nc)') ']: Voltages Vd (g), Vout\_f Ctrl 0 (' color(1) ') and Vout\_f Ctrl n (' color(2) ')'])
ylabel('Vout, Vd, Vout\_f')
xlabel('Time [s]')
grid on
%%%%%%%%%%%%
subplot(2,1,2); hold on           
title(['Vm = [' num2str(Vm(1:nc)') ']: Errors Vd-Vout\_f Ctrl 0 (' color(1) ') and Vd-Vout\_f Ctrl n (' color(2) ')'])
ylabel('Ve [V]')
xlabel('Time [s]')
grid on
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NrFig    = 4;
Fig_Name = 'Output_Voltage';
hh       = figure(NrFig); clf;
set(hh,'NumberTitle','off') % Clear the title
set(hh,'Name',[num2str(NrFig) '.' Fig_Name])
set(hh,'PaperPositionMode','manual','PaperPosition',[5 0 20 10]);
%%%%%%%%%%%%
subplot(2,1,1); 
hold on; grid on; zoom on
plot(t_in,Vd_t_in*Vin,'g','LineWidth',Lw1);
%%%%%%%%%%%%
subplot(2,1,2); 
hold on; grid on; zoom on
plot(t_in,Vd_t_in*Vin,'g','LineWidth',Lw1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ss_Range = 1:2;         
for ss = ss_Range
    LineSt_ss = [];
    for ll=1:ss
        LineSt_ss=[LineSt_ss LineSt];
    end
    if ss==1
        Select = 1;                      % Use control "Distance_0"
    else
        Select = 2;                      % Use control "Distance_n"
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    sim('Multilevel_Flying_Capacitor_Converter_mdl', Tfin);   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    figure(2);
    %%%%%%%%%%%%%
    subplot(1,2,1);        
    if nc==3
        plot(Vc(:,2),Vc(:,3),'Color',color(ss),'LineStyle',LineSt_ss,'LineWidth',Lw2);
    elseif nc==4
        plot3(Vc(:,2),Vc(:,3),Vc(:,4),'Color',color(ss),'LineStyle',LineSt_ss,'LineWidth',Lw2);
    end
    %%%%%%%%%%%%%
    subplot(1,2,2);    
    plot(t,vecnorm(Vc(:,2:end)-Vm0(2:end)',2,2)','Color',color(ss),'LineStyle',LineSt_ss,'LineWidth',Lw2) 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    figure(3);
    subplot(2,1,1); hold on         
    Vout_2 = filtromedia(Vout,200);
    plot(t,Vout_2,'Color',color(ss),'LineStyle',LineSt_ss,'LineWidth',Lw1);   
    %%%%%%%%%%%%%
    subplot(2,1,2); hold on           
    plot(t_in,Vd_t_in*Vin-interp1(t,Vout_2,t_in),'Color',color(ss),'LineStyle',LineSt_ss,'LineWidth',Lw0);
    %%%%%%%%%%%%%
    if ss==2
        ax(1)=subplot(2,1,1);
        ax(2)=subplot(2,1,2);
        linkaxes(ax,'x')
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    figure(4);
    subplot(2,1,ss)
    plot(t,Vout,'Color',[1 1 1]*0.7,'LineWidth',Lw0);
    plot(t,Vout_2,'Color',color(ss),'LineStyle','--','LineWidth',Lw1);
    ylim([-0.1 1.1]*Vin)
    set(gca,'YTick',(0:1/(m-1)*Vin:Vin))
    xx={};
    for tt=0:m-1
        if tt==0
            xx{tt+1,1}='0';
        elseif tt==m-1
            xx{tt+1,1}='1';
        else
            xx{tt+1,1}=[num2str(tt) '/' num2str(m-1)];
        end
    end
    set(gca,'YTickLabel',xx)
    %%%%%%%%%%
    if ss==1
        title(['Vm = [' num2str(Vm(1:nc)') ']. Voltages Vout (gr), Vd (g) and Vout\_f Ctrl 0 (' color(ss) ')'] )
    elseif ss==2
        title(['Vm = [' num2str(Vm(1:nc)') ']. Voltages Vout (gr), Vd (g) and Vout\_f Ctrl n (' color(ss) ')'] )
    end
    ylabel('Vout, Vd, Vout\_f')
    xlabel('Time [s]')
    %%%%%%%%%%%%%
    if ss==2
        ax(1)=subplot(2,1,1);
        ax(2)=subplot(2,1,2);
        linkaxes(ax,'x')
    end
end


















